@@ -32,7 +32,7 @@ def loadRulesFromFile(filename):
         # load jsonData from global, reduce disk usage
         return jsonData
     
-    print("Loading JSON file:", filename)
+    print("Loading JSON file: %s" % filename)
     
     with open(filename, 'r') as jsonf:
         jsonData = json.loads(jsonf.read())
@@ -43,7 +43,6 @@ class Freeway(object):
     """
     Freeway main class
     """
-    
     def __init__(self, filepath=None, pattern=['auto'], rules=None,
                  convertionTable=None, rulesfile=None, **kwargs):
 
@@ -54,16 +53,24 @@ class Freeway(object):
                 "No RULESFILE env variable was detected."
 
         self._rulesfile = rulesfile
-        jsonData = loadRulesFromFile(rulesfile)
-        self._rules = OrderedDict(Freeway.get_rules(jsonData['rules']))
-        self._convertionTable = jsonData['convertionTable']
+        
+        if not rules:
+            jsonData = loadRulesFromFile(rulesfile)
+            rules = jsonData.get('rules', None)
+  
+            if not convertionTable:
+                convertionTable = jsonData.get('convertionTable', None)
+
+        self._rules = OrderedDict(Freeway.get_rules(rules))
+        self._convertionTable = convertionTable
 
         for key, value in kwargs.items():
             setattr(self, key, value)
 
         if filepath:
             self._filepath = filepath.replace('\\', '/')
-            self.pattern = pattern
+        
+        self.pattern = pattern
 
     def __str__(self):
         return '%s: %s' % (self.pattern, str(self.data))
@@ -71,29 +78,41 @@ class Freeway(object):
     def __repr__(self):
         return str(self)
 
-    @staticmethod
-    def info_from_path(path, rules, patterns=['auto']):
+    def info_from_path(self, firstMatch=True):
         """
         Parse a path with pattern rules.
         """
-        assert isinstance(path, str), 'Path isnt str type'
+        patterns = list(self._rules.keys()) if self._pattern == ["auto"] else self._pattern
+
+        self._pattern = []
+
         for pattern in patterns:
-            if pattern == 'auto':
-                for key in rules:
-                    for item in Freeway.expandRules(key, Freeway(rules=rules)):
-                        match = re.match(item.regex, path, re.IGNORECASE)
-                        if match:
-                            for data in Freeway.info_from_path(path,
-                                                               rules,
-                                                               [item.name]):
-                                yield data
-            else:
-                for item in Freeway.expandRules(pattern, Freeway(rules=rules)):
-                    match = re.match(item.regex, path, re.IGNORECASE)
-                    if match:
-                        info = {pattern: value for pattern, value in match.groupdict(
-                        ).items() if not pattern.endswith('_')}
-                        yield info
+            data = self.resolve_pattern(pattern)
+            if data:
+                yield data
+                self._pattern.append(pattern)
+                if firstMatch:
+                    break
+
+    def ensureCompiledRegex(self, pattern):
+        for rule in self._rules.get(pattern):
+            if not rule.compiled:
+                rules = list(self.expandRules(pattern))
+                self._rules[pattern] = rules
+        
+    def resolve_pattern(self, pattern):
+        resolved = {}
+
+        self.ensureCompiledRegex(pattern)
+                
+        for rule in self._rules.get(pattern):
+            match = rule.compiled.match(self._filepath)
+
+            if match:
+                resolved.update({pattern: value for pattern, value in match.groupdict(
+                ).items() if not pattern.endswith('_')})
+
+        return resolved
 
     @property
     def match(self):
@@ -103,7 +122,7 @@ class Freeway(object):
         patterns = {}
         fullMatch = False
         for pattern in self.pattern:
-            for rule in Freeway.expandRules(pattern, Freeway(rules=self._rules)):
+            for rule in self.expandRules(pattern):
                 for field in rule.fields:
                     if self.data.get(field):
                         patterns[pattern] = True
@@ -119,6 +138,12 @@ class Freeway(object):
 
         return fullMatch
 
+    def _parsePath(self):
+        if self._filepath:
+            for find in self.info_from_path():
+                for key, value in find.items():
+                    setattr(self, key, value)
+                    
     @property
     def pattern(self):
         """
@@ -132,13 +157,23 @@ class Freeway(object):
         Setting a pattern and parse new fields.
         """
         self._pattern = [value] if isinstance(value, str) else value
-        if self._filepath:
-            for find in Freeway.info_from_path(self._filepath,
-                                               self._rules,
-                                               self._pattern):
-                for key, value in find.items():
-                    setattr(self, key, value)
-
+        self._parsePath()
+        
+    @property
+    def filepath(self):
+        """
+        Current filepath in use.
+        """
+        return self._filepath
+    
+    @filepath.setter
+    def filepath(self, value):
+        """
+        Setting a filepath and parse new fields.
+        """
+        self._filepath = value
+        self._parsePath()
+        
     @property
     def data(self):
         """
@@ -146,7 +181,8 @@ class Freeway(object):
         """
         
         elements = self.__dict__.copy()
-        for attr in ['pattern', '_pattern', '_filepath', '_rules',
+        for attr in ['pattern', '_pattern',
+                     'filepath', '_filepath', '_rules',
                      '_convertionTable', '_rulesfile']:
             elements.pop(attr, None)
 
@@ -161,24 +197,24 @@ class Freeway(object):
             if not name.startswith('_'):
                 yield name, [RuleParser(name, rule) for rule in rules]
 
-    @staticmethod
-    def expandRules(attr, rules):
+    def expandRules(self, attr):
         """
         Evaluate recursively a pattern rule and return a full regex
         """
-        rules._rules["_ignoreMissing"] = True
-
-        for rule in rules._rules.get(attr, []):
+        rules = self._rules.copy()
+        self._rules["_ignoreMissing"] = True
+        
+        for rule in rules.get(attr, []):
             rule = RuleParser(attr, str(rule))
-            while set(rules._rules) & set(rule.fields):
+            while set(rules) & set(rule.fields):
                 for field in rule.fields:
                     if field in rules:
                         rule.rule = rule.rule.replace('{%s}' % field,
-                                                      rules.get(field, field))
-
+                                                      rules.get(field, field)[0].rule)
+            rule.compile(re.IGNORECASE)
             yield rule
 
-        rules._rules["_ignoreMissing"] = False
+        self._rules["_ignoreMissing"] = False
 
     def __getattribute__(self, attr):
         ignoreMissing = object.__getattribute__(
@@ -188,25 +224,16 @@ class Freeway(object):
         if rules:
             for rule in rules:
                 try:
-                    name = rule.rule
-                    for field in rule.fields:
-                        value = getattr(self, field, None)
-                        if value is not None:
-                            name = name.replace('{%s}' % field, value)
-                        else:
-                            break
+                    data = {field: getattr(self, field, None) for field in rule.fields}
+                    missing = [k for k, v in data.items() if v is None]
+                    if missing:
+                        raise AttributeError
 
-                    if not _PLACEHOLDER_REGEX.findall(name):
-                        break
+                    name = rule.rule.format(**data)
 
                 except AttributeError:
-                    raise Exception("Can't find '" + field + "' attribute.")
-
-            missing = [part for part in _PLACEHOLDER_REGEX.findall(name)]
-
-            if missing and not ignoreMissing:
-                raise AttributeError(
-                    'No se ha encontrado el atributo: %s' % ', '.join(missing))
+                    if not ignoreMissing:
+                        raise Exception("Can't find [%s] attribute(s)." % ', '.join(missing))
 
             return name
         else:
@@ -223,7 +250,6 @@ class Freeway(object):
                                 value = object.__getattribute__(self, key)
                                 index = switchs[table][key].index(value)
                                 return switchs[table][attr][index]
-
                             except (AttributeError, ValueError):
                                 pass
 
@@ -243,6 +269,10 @@ class Freeway(object):
     def update(self, data):
         self.__dict__.update(data)
 
+    def copy(self):
+        return Freeway(self.filepath, pattern=self.pattern, rules=self._rules,
+                       convertionTable=self._convertionTable, **self.data)
+
     def clean(self):
         notRemove = ['pattern', '_rules', '_convertionTable', '_rulesfile']
         for key in set(self.__dict__) ^ set(notRemove):
@@ -259,6 +289,7 @@ class RuleParser(object):
     def __init__(self, name, rule):
         self.name = name
         self.rule = str(rule)
+        self.compiled = None
 
     def __getitem__(self, item):
         index = 0
@@ -297,6 +328,9 @@ class RuleParser(object):
 
     @property
     def regex(self):
+        if "_regex" in self.__dict__:
+            return self._regex
+
         duplis = []
         regexRule = self.rule
         for field in _FIELDPATTERN_REGEX.findall(self.rule):
@@ -309,15 +343,8 @@ class RuleParser(object):
             regexRule = regexRule.replace(
                 field, _FIELDREPLACE % fieldReplace, 1)
 
-        return '' + regexRule
-
-
-if __name__ == '__main__':
-    # "assetFile": ["{project}_{assetType}_{asset}_{task}_v.{version}.{ext}"]
+        self._regex = "^%s$" % regexRule
+        return self._regex
     
-    ruta = r"example_Character_Vidi_MOD_v001.ma"
-    path = r"C:/example/assets/Character/Vidi/work/example_Character_Vidi_MOD_v001.ma"
-    myPath = Freeway(path)
-    print(myPath)
-    
-    print(myPath.assetPath)
+    def compile(self, flag):
+        self.compiled = re.compile(self.regex, flag)
